#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/wait.h>

void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}
/* Program demonstrira exec sistemski poziv koriscenjem 2 od 6 funkcija iz 
   familije exec funkcija. */
int main(){
  pid_t pid;

  if ( (pid = fork()) < 0)
    error_fatal ("first fork error");
  else if (pid == 0){
    /* Dete proces se zamenjuje programom args cija je putanja navedena.
       Program args kao argumente prima
       argv[0] = "args"
       argv[1] = "arg1"
       argv[2] = "arg2"
       Pri pozivu bilo koje od exec funkcija sa sufiksom 'l' ne treba zaboraviti da je prvi
       argument posle imena programa koji se pokrece ime pod kojim ce taj program videti sebe.
    */
    if (execl ("../args/args", "args", "arg1", "arg2", (char *)0) < 0)
      error_fatal ("execl error");
  }
  else {
    /* Roditelj ceka da se dete proces zavrsi.*/
    if (waitpid (pid, NULL, 0) < 0)
      error_fatal ("first waitpid error");
  }

  if ( (pid = fork()) < 0)
    error_fatal ("second fork error");
  else if (pid == 0){
    /* Pokusava se sa izvrsavanjem programa args pomocu execlp
       funkcije. Posto je navedeno slovo 'p' kao sufiks, a ime
       programa ne sadrzi karakter '/' trazenje programa args se
       izvrsava u direktorijumima navedenim u PATH promenljivoj.
       Posto se direktorijum ../args/ ne nalazi u PATH-u, bice
       prijavljena greska. Ukoliko bi se navela putanja koja sadrzi
       '/', npr. ../args, onda bi execlp ignorisao PATH i pokusao
       preko putanje da pronadje args - u ovom slucaju bi uspeo.*/
    if (execlp ("args", "args", "arg1", "arg2", (char *)0) < 0)
      error_fatal ("execlp error");
  }
  else {
    /* Roditelj ceka da se dete proces zavrsi.*/
    if (waitpid (pid, NULL, 0) < 0)
      error_fatal ("second waitpid error");
  }

  return 0;
}

